@@ -508,10 +508,19 @@ async_call_send (AsyncDBusCall *async_call)
 {
   AsyncCallCancelData *cancel_data;
 
+  _g_dbus_connection_call_async (async_call->connection,
+				 async_call->message,
+				 G_VFS_DBUS_TIMEOUT_MSECS,
+				 async_dbus_response,
+				 async_call);
+
   if (async_call->cancellable)
     {
       cancel_data = g_new0 (AsyncCallCancelData, 1);
       cancel_data->connection = dbus_connection_ref (async_call->connection);
+      /* make sure we get the serial *after* the message has been sent, otherwise
+       * it will be 0
+       */
       cancel_data->serial = dbus_message_get_serial (async_call->message);
       async_call->cancelled_tag =
 	g_signal_connect_data (async_call->cancellable, "cancelled",
@@ -520,12 +529,6 @@ async_call_send (AsyncDBusCall *async_call)
 			       (GClosureNotify)async_call_cancel_data_free,
 			       0);
     }
-
-  _g_dbus_connection_call_async (async_call->connection,
-				 async_call->message,
-				 G_VFS_DBUS_TIMEOUT_MSECS,
-				 async_dbus_response,
-				 async_call);
 }
 
 static void
@@ -801,6 +804,7 @@ _g_vfs_daemon_call_sync (DBusMessage *message,
 
       reply = dbus_pending_call_steal_reply (pending);
       dbus_pending_call_unref (pending);
+      g_cancellable_release_fd (cancellable);
     }
   else
     {
@@ -1021,3 +1025,28 @@ _g_dbus_connection_get_sync (const char *dbus_id,
 
   return connection;
 }
+
+/**
+ * _g_simple_async_result_complete_with_cancellable:
+ * @result: the result
+ * @cancellable: a cancellable to check
+ *
+ * If @cancellable is cancelled, sets @result into the cancelled error
+ * state. Then calls g_simple_async_result_complete().
+ * This function is useful to ensure that @result is properly set into
+ * an error state on cancellation.
+ **/
+void
+_g_simple_async_result_complete_with_cancellable (GSimpleAsyncResult *result,
+                                                  GCancellable       *cancellable)
+{
+  if (cancellable &&
+      g_cancellable_is_cancelled (cancellable))
+    g_simple_async_result_set_error (result,
+                                     G_IO_ERROR,
+                                     G_IO_ERROR_CANCELLED,
+                                     "%s", _("Operation was cancelled"));
+
+  g_simple_async_result_complete (result);
+}
+
