@@ -1378,6 +1378,11 @@ soup_authenticate_interactive (SoupSession *session,
 
   if (res && !aborted)
     {
+      /* it's not safe to assume that we get the username filed in,
+         in the case that we provied a default username */
+      if (new_username == NULL)
+        new_username = g_strdup (info->username);
+
       soup_auth_authenticate (auth, new_username, new_password);
 
       g_free (info->username);
@@ -2301,7 +2306,9 @@ do_set_display_name (GVfsBackend           *backend,
 
 static gboolean
 try_unmount (GVfsBackend    *backend,
-             GVfsJobUnmount *job)
+             GVfsJobUnmount *job,
+             GMountUnmountFlags flags,
+             GMountSource *mount_source)
 {
   _exit (0);
 }
